/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.beans.application;

import java.io.File;
import java.util.List;

import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.impl.MAnalysisMetaModelFactory;
import kieker.webgui.beans.session.SelectedProjectBean;
import kieker.webgui.common.FileManager;

import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;

/**
 * This bean can be used to handle all available projects within the program.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@ApplicationScoped
public class AvailableProjectsBean {

	/**
	 * A list containing all available projects.
	 */
	private final List<MIProject> projects;
	/**
	 * The factory used to create new projects.
	 */
	private final MAnalysisMetaModelFactory factory;

	/**
	 * Creates a new instance of this class.
	 */
	public AvailableProjectsBean() {
		this.projects = FileManager.getInstance().loadAllProjects();
		this.factory = new MAnalysisMetaModelFactory();
	}

	/**
	 * Uses the given name to create a new project. If a project with the
	 * same name does already exist, nothing happens.
	 * 
	 * @param projectName
	 *            The name of the new project.
	 */
	public final void addProject(final String projectName) {
		synchronized (this) {
			/*
			 * Create a new project.
			 */
			final MIProject project = this.factory.createProject();
			project.setName(projectName);

			/*
			 * Try to save the project.
			 */
			if (FileManager.getInstance().saveNewProject(project)) {
				/*
				 * Set the new project as the main project, if the current
				 * user doesn't have one.
				 */
				final FacesContext context = FacesContext.getCurrentInstance();
				final SelectedProjectBean bean = context.getApplication().evaluateExpressionGet(context, "#{selectedProjectBean}",
						SelectedProjectBean.class);
				if (bean != null && bean.getMainProject() == null) {
					bean.setMainProject(project);
				}

				this.projects.add(project);
			}
		}
	}

	/**
	 * Delivers the currently available projects as a tree root.
	 * 
	 * @return The root of the currently available projects.
	 */
	public final TreeNode getProjectsRoot() {
		synchronized (this) {
			final TreeNode root = new DefaultTreeNode("Root", null);

			for (final MIProject project : this.projects) {
				final TreeNode projectNode = new DefaultTreeNode("project", project, root);
				final TreeNode dependenciesNode = new DefaultTreeNode("dependencies", "Dependencies", projectNode);
				final TreeNode usedPluginsNode = new DefaultTreeNode("usedPlugins", "Used Plugins", projectNode);

				for (final MIPlugin plugin : project.getPlugins()) {
					new DefaultTreeNode("usedPlugin", plugin.getClassname(), usedPluginsNode);
				}
				for (final MIDependency dependency : project.getDependencies()) {
					new DefaultTreeNode("dependencies", new File(dependency.getFilePath()).getName(), dependenciesNode);
				}
			}

			return root;
		}
	}

	/**
	 * This method saved a given project on the file system. The project
	 * should already exist.
	 * 
	 * @param project
	 *            The project to be saved.
	 */
	public void saveProject(final MIProject project) {
		synchronized (this) {
			FileManager.getInstance().saveProject(project);
		}
	}

	/**
	 * Deletes a given project from the file system.
	 * 
	 * @param project
	 *            The project to be removed.
	 */
	public void deleteProject(final MIProject project) {
		synchronized (this) {
			final boolean result = FileManager.getInstance().deleteProject(project);
			if (result) {
				this.projects.remove(project);
			}
		}
	}

	/**
	 * Reloads a given project from the file system. In other words: The
	 * state of the project is restored from the file system.
	 * 
	 * @param project
	 *            The project to be reloaded.
	 */
	public void resetProject(final MIProject project) {
		synchronized (this) {
			final MIProject result = FileManager.getInstance().reloadProject(project);
			if (result != null) {
				this.projects.remove(project);
				this.projects.add(result);
			}
		}
	}
}
