/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.converter;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.webgui.common.PluginFinder;

/**
 * This converter can be used to convert an instance of <i>MIDependency</i> to
 * the number of plugins within the dependency, but of course <b>not</b> vice
 * versa. The number is cached to avoid that the converter holds the file
 * permanently.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@FacesConverter(value = MIDependencyToCountPluginsConverter.NAME)
public class MIDependencyToCountPluginsConverter implements Converter {

	/**
	 * The name of this class used by the xhtml-files.
	 */
	public static final String NAME = "kieker.webgui.converter.MIDependencyToCountPluginsConverter";

	/**
	 * The maximal number of dependencies stored within the cache.
	 */
	private static final int MAX_CACHE_SIZE = 20;

	/**
	 * This map is the cache and contains the number of plugins within the
	 * dependencies.
	 */
	private final Map<MIDependency, String> cache;

	/**
	 * Creates a new instance of this class.
	 */
	public MIDependencyToCountPluginsConverter() {
		this.cache = new HashMap<MIDependency, String>();
	}

	/**
	 * Delivers always null.
	 * 
	 * @param fc
	 *            The FacesContext for the request being processed.
	 * @param uic
	 *            The component with which this model object value is
	 *            associated.
	 * @param string
	 *            The string to be converted.
	 * @return Always null.
	 */
	@Override
	public Object getAsObject(final FacesContext fc, final UIComponent uic, final String string) {
		return null;
	}

	/**
	 * Converts the given dependency to a human readable string
	 * containing the number of plugins within the jar.
	 * 
	 * @param fc
	 *            The FacesContext for the request being processed.
	 * @param uic
	 *            The component with which this model object value is
	 *            associated.
	 * @param o
	 *            The object to be converted.
	 * @return The number of plugins within the jar. If this is
	 *         not possible, an empty string will be returned.
	 */
	@Override
	public String getAsString(final FacesContext fc, final UIComponent uic, final Object o) {
		if (!(o instanceof MIDependency)) {
			return "";
		} else {
			/* Try to find the library within the cache. */
			if (this.cache.containsKey(o)) {
				return this.cache.get(o);
			}
			/* If the cache is too big, remove one of the objects. */
			if (this.cache.size() > MIDependencyToCountPluginsConverter.MAX_CACHE_SIZE) {
				this.cache.remove(this.cache.keySet().iterator().next());
			}
			String result;
			try {
				result = Integer.toString(PluginFinder.getAllPluginsWithinJar(new URL("file", "localhost", ((MIDependency) o).getFilePath())).size());
			} catch (final MalformedURLException ex) {
				result = "";
			} catch (final NullPointerException ex) {
				result = "";
			}
			this.cache.put((MIDependency) o, result);
			return result;
		}
	}
}
